"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault")["default"];
var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard")["default"];
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Internal = void 0;
var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));
var _react = _interopRequireWildcard(require("react"));
var _reactCodemirror = require("@uiw/react-codemirror");
var _merge = require("@codemirror/merge");
var _store = require("./store");
var _jsxRuntime = require("react/jsx-runtime");
var _excluded = ["className", "children", "orientation", "revertControls", "highlightChanges", "gutter", "collapseUnchanged", "renderRevertControl"],
  _excluded2 = ["modified", "modifiedExtension", "original", "originalExtension", "theme", "view", "dispatch"];
var Internal = /*#__PURE__*/_react["default"].forwardRef(function (props, ref) {
  var className = props.className,
    children = props.children,
    orientation = props.orientation,
    revertControls = props.revertControls,
    highlightChanges = props.highlightChanges,
    gutter = props.gutter,
    collapseUnchanged = props.collapseUnchanged,
    renderRevertControl = props.renderRevertControl,
    elmProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var _useStore = (0, _store.useStore)(),
    modified = _useStore.modified,
    modifiedExtension = _useStore.modifiedExtension,
    original = _useStore.original,
    originalExtension = _useStore.originalExtension,
    theme = _useStore.theme,
    view = _useStore.view,
    dispatch = _useStore.dispatch,
    otherStore = (0, _objectWithoutProperties2["default"])(_useStore, _excluded2);
  var editor = (0, _react.useRef)(null);
  var opts = {
    orientation: orientation,
    revertControls: revertControls,
    highlightChanges: highlightChanges,
    gutter: gutter,
    collapseUnchanged: collapseUnchanged,
    renderRevertControl: renderRevertControl
  };
  (0, _react.useImperativeHandle)(ref, function () {
    return {
      container: editor.current,
      view: view,
      modified: modified,
      original: original,
      config: (0, _objectSpread2["default"])({
        a: original,
        b: modified,
        parent: editor.current
      }, opts)
    };
  }, [editor, view, modified, original, opts]);
  (0, _react.useEffect)(function () {
    if (view && original && modified && theme && editor.current && dispatch) {
      editor.current.innerHTML = '';
      new _merge.MergeView((0, _objectSpread2["default"])({
        a: (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, original), {}, {
          extensions: [].concat((0, _toConsumableArray2["default"])(originalExtension || []), (0, _toConsumableArray2["default"])((0, _reactCodemirror.getDefaultExtensions)({
            theme: theme
          })))
        }),
        b: (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, modified), {}, {
          extensions: [].concat((0, _toConsumableArray2["default"])(modifiedExtension || []), (0, _toConsumableArray2["default"])((0, _reactCodemirror.getDefaultExtensions)({
            theme: theme
          })))
        }),
        parent: editor.current
      }, opts));
    }
  }, [theme, editor.current, original, modified, originalExtension, modifiedExtension]);
  (0, _react.useEffect)(function () {
    if (!view && editor.current && original !== null && original !== void 0 && original.extensions && modified !== null && modified !== void 0 && modified.extensions) {
      var viewDefault = new _merge.MergeView((0, _objectSpread2["default"])({
        a: original,
        b: modified,
        parent: editor.current
      }, opts));
      dispatch && dispatch((0, _objectSpread2["default"])({
        view: viewDefault,
        container: editor.current
      }, opts));
    }
  }, [editor.current, original, modified, view]);
  (0, _react.useEffect)(function () {
    return function () {
      return view && view.destroy();
    };
  }, []);
  (0, _react.useEffect)(function () {
    if (view) {
      var _opts = {};
      if (otherStore.orientation !== orientation) {
        _opts.orientation = orientation;
      }
      if (otherStore.revertControls !== revertControls) {
        _opts.revertControls = revertControls;
      }
      if (otherStore.highlightChanges !== highlightChanges) {
        _opts.highlightChanges = highlightChanges;
      }
      if (otherStore.gutter !== gutter) {
        _opts.gutter = gutter;
      }
      if (otherStore.collapseUnchanged !== collapseUnchanged) {
        _opts.collapseUnchanged = collapseUnchanged;
      }
      if (Object.keys(_opts).length && dispatch && original && modified && editor.current) {
        view.destroy();
        var viewDefault = new _merge.MergeView((0, _objectSpread2["default"])({
          a: original,
          b: modified,
          parent: editor.current
        }, _opts));
        dispatch((0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, _opts), {}, {
          renderRevertControl: renderRevertControl,
          view: viewDefault
        }));
      }
    }
  }, [view, original, modified, editor, orientation, revertControls, highlightChanges, gutter, collapseUnchanged, renderRevertControl]);
  var defaultClassNames = 'cm-merge-theme';
  return /*#__PURE__*/(0, _jsxRuntime.jsx)("div", (0, _objectSpread2["default"])((0, _objectSpread2["default"])({
    ref: editor,
    className: "".concat(defaultClassNames).concat(className ? " ".concat(className) : '')
  }, elmProps), {}, {
    children: children
  }));
});
exports.Internal = Internal;
Internal.displayName = 'CodeMirrorMerge.Internal';